@@ -1,6 +1,6 @@
 <?php
-/*
- * $Horde: nag/index.php,v 1.7 2001/10/21 21:37:11 chuck Exp $
+/**
+ * $Horde: nag/index.php,v 1.7.2.2 2002/12/17 03:46:12 jon Exp $
  *
  * Copyright 2001 Jon Parise <jon@horde.org>
  *
@@ -13,20 +13,9 @@ $nag_configured = (@is_readable(NAG_BASE . '/config/conf.php') &&
                    @is_readable(NAG_BASE . '/config/html.php') &&
                    @is_readable(NAG_BASE . '/config/prefs.php'));
 
-if ($nag_configured) {
-    include_once NAG_BASE . '/lib/base.php';
-    
-    $uri = 'list.php';
-    if (!empty($HTTP_SERVER_VARS['QUERY_STRING'])) {
-        $uri .= '?' . $HTTP_SERVER_VARS['QUERY_STRING'];
-    }
-    
-    header('Location: ' . Horde::applicationUrl($uri, true));
+if (!$nag_configured) {
+    require NAG_BASE . '/templates/index/notconfigured.inc';
     exit;
-    
-/* Nag isn't configured */
-} else {
-    include NAG_BASE . '/templates/index/notconfigured.inc';
 }
 
-?>
+require NAG_BASE . '/list.php';
