@@ -1,5 +1,5 @@
 <?php
-// $Horde: nag/lib/Nag.php,v 1.39.2.3 2002/06/06 12:51:47 bjn Exp $
+// $Horde: nag/lib/Nag.php,v 1.39.2.13 2002/12/17 04:28:00 jon Exp $
 
 require_once NAG_BASE . '/lib/constants.php';
 
@@ -14,8 +14,10 @@ $completion_map = array(
 /**
  * Nag Base Class.
  *
+ * $Horde: nag/lib/Nag.php,v 1.39.2.13 2002/12/17 04:28:00 jon Exp $
+ *
  * @author Jon Parise <jon@horde.org>
- * @version $Revision: 1.39.2.3 $
+ * @version $Revision: 1.39.2.13 $
  * @package nag
  */
 class Nag {
@@ -52,7 +54,7 @@ class Nag {
             }
         }
 
-        /* Sort the task list. */
+        /* Sorting criteria for the task list. */
         $sort_functions = array(
             NAG_SORT_PRIORITY => 'ByPriority',
             NAG_SORT_NAME => 'ByName',
@@ -64,7 +66,7 @@ class Nag {
         /* Sort the array if we have a sort function defined for this field. */
         if (isset($sort_functions[$sortby])) {
             $prefix = ($sortdir == NAG_SORT_DESCEND) ? '_rsort' : '_sort';
-            uasort($tasks, $prefix . $sort_functions[$sortby]);
+            uasort($tasks, array('Nag', $prefix . $sort_functions[$sortby]));
         }
 
         return $tasks;
@@ -78,7 +80,7 @@ class Nag {
     function listCategories()
     {
         global $prefs;
-        
+
         static $catString, $categories;
 
         if (!isset($categories) || $catString != $prefs->getValue('task_categories')) {
@@ -94,6 +96,119 @@ class Nag {
         return $categories;
     }
 
+    /**
+     * Add a new category
+     *
+     * @param string  $name     The name of the category to add.
+     *
+     * @return integer          A valid category id, 0 on failure or
+     *                          the new category's id on success.
+     */
+    function addCategory($name)
+    {
+        global $prefs;
+
+        if ($prefs->isLocked('task_categories') || empty($name)) {
+            return 0;
+        }
+
+        $categories = Nag::listCategories();
+
+        if (in_array($name, $categories)) {
+            return 0;
+        }
+
+        $categories[] = $name;
+        unset($categories[0]);
+
+        $cats = array();
+        $key = 0;
+        foreach ($categories as $key => $cat) {
+            $cat = array($key, $cat);
+            $cats[] = implode(':', $cat);
+        }
+
+        $catString = implode('|', $cats);
+        $prefs->setValue('task_categories', $catString);
+        $prefs->store();
+
+        return $key;
+    }
+
+    /**
+     * Delete a category
+     *
+     * @param integer   $categoryID The id of the category to remove.
+     *
+     * @return boolean              True on success, false on failure.
+     */
+    function deleteCategory($categoryID)
+    {
+        global $prefs;
+        $categories = Nag::listCategories();
+
+        if ($prefs->isLocked('task_categories') || !array_key_exists($categoryID, $categories)) {
+            return false;
+        }
+
+        /* The first category ("Unfiled") cannot be deleted. */
+        unset($categories[0]);
+
+        /* Delete the specified category. */
+        unset($categories[$categoryID]);
+
+        $cats = array();
+        foreach ($categories as $key => $cat) {
+            $cat = array($key, $cat);
+            $cats[] = implode(':', $cat);
+        }
+
+        $catString = implode('|', $cats);
+        $prefs->setValue('task_categories', $catString);
+        $prefs->store();
+
+        return true;
+    }
+
+
+    /**
+     * Rename a category
+     *
+     * @param integer   $categoryID The id of the category to remove.
+     * @param string    $name       The new name of the category.
+     *
+     * @return boolean              True on success, false on failure.
+     */
+    function renameCategory($categoryID, $name)
+    {
+        global $prefs;
+        $categories = Nag::listCategories();
+
+        if ($prefs->isLocked('task_categories') || empty($name) ||
+                        !array_key_exists($categoryID, $categories) ||
+                        in_array($name, $categories)) {
+            return false;
+        }
+
+        /* The first category ("Unfiled") cannot be renamed. */
+        unset($categories[0]);
+
+        /* Assign the new name to the specified category. */
+        $categories[$categoryID] = $name;
+
+        $cats = array();
+        foreach ($categories as $key => $cat) {
+            $cat = array($key, $cat);
+            $cats[] = implode(':', $cat);
+        }
+
+        $catString = implode('|', $cats);
+        $prefs->setValue('task_categories', $catString);
+        $prefs->store();
+
+        return true;
+    }
+
     /**
      * Generates the HTML for a day selection widget.
      *
@@ -143,7 +258,7 @@ class Nag {
         for ($month = 1; $month <= 12; $month++) {
             $html .= "<option value=\"$month\"";
             $html .= ($month == $selected) ? ' selected="selected">' : '>';
-            $html .= strftime('%B', mktime(0, 0, 0, $month)) . '</option>';
+            $html .= strftime('%B', mktime(0, 0, 0, $month, 1)) . '</option>';
         }
         $html .= "</select>\n";
 
@@ -247,19 +362,20 @@ class Nag {
      */
     function buildPriorityWidget($name, $selected = -1)
     {
-        $html = "<select name=\"$name\">";
+        $descs = array(1 => _("(highest)"), 5 => _("(lowest)"));
 
+        $html = "<select name=\"$name\">";
         for ($priority = 1; $priority <= 5; $priority++) {
             $html .= "<option value=\"$priority\"";
             $html .= ($priority == $selected) ? ' selected="selected">' : '>';
-            $html .= $priority . '</option>';
+            $html .= $priority . ' ' . @$descs[$priority] . '</option>';
         }
         $html .= "</select>\n";
 
         return $html;
     }
 
-    /**                                                                        
+    /**
      * Builds the HTML for a task completion state widget.
      *
      * @param string  $name         The name of the widget.
@@ -283,21 +399,30 @@ class Nag {
         return $html;
     }
 
-    /**                                                                        
+    /**
      * Builds the HTML for a task category widget.
      *
-     * @param string  $name         The name of the widget.
-     * @param integer $selected     (optional) The default category.
+     * @param string  $name       The name of the widget.
+     * @param integer $selected   (optional) The default category.
+     * @param boolean $newheader  (optional) Include a new new category option.
      *
      * @return string       The HTML <select> widget.
      */
-    function buildCategoryWidget($name, $selected = 0)
+    function buildCategoryWidget($name, $selected = false, $newheader = false)
     {
         $html = "<select name=\"$name\">";
 
+        $html .= '<option value="">'. _("Select Category") . '</option>';
+
+        if ($newheader) {
+            $html .= '<option value="">----</option>' . "\n";
+            $html .= '<option value="*new*">' . _("New Category") . "</option>\n";
+            $html .= '<option value="">----</option>' . "\n";
+        }
+
         foreach (Nag::listCategories() as $id => $name) {
             $html .= "<option value=\"$id\"";
-            $html .= ($id == $selected) ? ' selected="selected">' : '>';
+            $html .= ($id == $selected && $selected !== false) ? ' selected="selected">' : '>';
             $html .= $name . '</option>';
         }
         $html .= '</select>';
@@ -363,142 +488,171 @@ class Nag {
 
         return _("Unknown");
     }
-}
 
-/**
- * Comparison function for sorting tasks by priority.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
- */
-function _sortByPriority($a, $b)
-{
-    if ($a['priority'] == $b['priority']) return 0;
-    return ($a['priority'] > $b['priority']) ? 1 : -1;
-}
+    /**
+     * Add a name=value pair to an URL, taking care of whether there
+     * are existing parameters and whether to use ? or & as the glue.
+     *
+     * @access public
+     *
+     * @param string $url       The URL to modify
+     * @param string $parameter The name=value pair to add.
+     *
+     * @return string The modified URL.
+     *
+     * @since Nag 1.1
+     */
+    function addParameter($url, $parameter)
+    {
+        if (!empty($parameter) && strstr($url, $parameter) === false) {
+            if (substr($parameter, 0, 1) == '?') {
+                $parameter = substr($parameter, 1);
+            }
 
-/**
- * Comparison function for reverse sorting tasks by priority.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
- */
-function _rsortByPriority($a, $b)
-{
-    if ($a['priority'] == $b['priority']) return 0;
-    return ($a['priority'] < $b['priority']) ? -1 : 1;
-}
+            $pos = strpos($url, '?');
+            if ($pos !== false) {
+                $url = substr_replace($url, $parameter . ini_get('arg_separator.output'),
+                                      $pos + 1, 0);
+            } else {
+                $url .= '?' . $parameter;
+            }
+        }
+        return $url;
+    }
 
-/**
- * Comparison function for sorting tasks by name.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
- */
-function _sortByName($a, $b)
-{
-    return strcmp($a['name'], $b['name']);
-}
+    /**
+     * Comparison function for sorting tasks by priority.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     */
+    function _sortByPriority($a, $b)
+    {
+        if ($a['priority'] == $b['priority']) return 0;
+        return ($a['priority'] > $b['priority']) ? 1 : -1;
+    }
 
-/**
- * Comparison function for reverse sorting tasks by name.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
- */
-function _rsortByName($a, $b)
-{
-    return strcmp($b['name'], $a['name']);
-}
+    /**
+     * Comparison function for reverse sorting tasks by priority.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     */
+    function _rsortByPriority($a, $b)
+    {
+        if ($a['priority'] == $b['priority']) return 0;
+        return ($a['priority'] > $b['priority']) ? -1 : 1;
+    }
 
-/**
- * Comparison function for sorting tasks by category.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
- */
-function _sortByCategory($a, $b)
-{
-    return strcmp($a['category'], $b['category']);
-}
+    /**
+     * Comparison function for sorting tasks by name.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     */
+    function _sortByName($a, $b)
+    {
+        return strcmp($a['name'], $b['name']);
+    }
 
-/**
- * Comparison function for reverse sorting tasks by category.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
- */
-function _rsortByCategory($a, $b)
-{
-    return strcmp($b['category'], $a['category']);
-}
+    /**
+     * Comparison function for reverse sorting tasks by name.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     */
+    function _rsortByName($a, $b)
+    {
+        return strcmp($b['name'], $a['name']);
+    }
 
-/**
- * Comparison function for sorting tasks by due date.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
- */
-function _sortByDue($a, $b)
-{
-    if ($a['due'] == $b['due']) return 0;
-    return ($a['due'] > $b['due']) ? -1 : 1;
-}
+    /**
+     * Comparison function for sorting tasks by category.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     */
+    function _sortByCategory($a, $b)
+    {
+        return strcmp($a['category'], $b['category']);
+    }
 
-/**
- * Comparison function for reverse sorting tasks by due date.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
- */
-function _rsortByDue($a, $b)
-{
-    if ($a['due'] == $b['due']) return 0;
-    return ($a['due'] < $b['due']) ? -1 : 1;
-}
+    /**
+     * Comparison function for reverse sorting tasks by category.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     */
+    function _rsortByCategory($a, $b)
+    {
+        return strcmp($b['category'], $a['category']);
+    }
 
-/**
- * Comparison function for sorting tasks by completion status.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
- */
-function _sortByCompletion($a, $b)
-{
-    if ($a['completed'] == $b['completed']) return 0;
-    return ($a['completed'] > $b['completed']) ? -1 : 1;
-}
+    /**
+     * Comparison function for sorting tasks by due date.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     */
+    function _sortByDue($a, $b)
+    {
+        if ($a['due'] == $b['due']) return 0;
+        return ($a['due'] > $b['due']) ? -1 : 1;
+    }
 
-/**
- * Comparison function for reverse sorting tasks by completion status.
- *
- * @param array $a  Task one.
- * @param array $b  Task two.
- *
- * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
- */
-function _rsortByCompletion($a, $b)
-{
-    if ($a['completed'] == $b['completed']) return 0;
-    return ($a['completed'] < $b['completed']) ? -1 : 1;
-}
+    /**
+     * Comparison function for reverse sorting tasks by due date.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     */
+    function _rsortByDue($a, $b)
+    {
+        if ($a['due'] == $b['due']) return 0;
+        return ($a['due'] < $b['due']) ? -1 : 1;
+    }
 
-?>
+    /**
+     * Comparison function for sorting tasks by completion status.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     */
+    function _sortByCompletion($a, $b)
+    {
+        if ($a['completed'] == $b['completed']) return 0;
+        return ($a['completed'] > $b['completed']) ? -1 : 1;
+    }
+
+    /**
+     * Comparison function for reverse sorting tasks by completion status.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     */
+    function _rsortByCompletion($a, $b)
+    {
+        if ($a['completed'] == $b['completed']) return 0;
+        return ($a['completed'] < $b['completed']) ? -1 : 1;
+    }
+}
