@@ -1,6 +1,6 @@
 <?php
-/*
- * $Horde: nag/prefs.php,v 1.14.2.3 2002/04/23 06:25:16 chuck Exp $
+/**
+ * $Horde: nag/prefs.php,v 1.14.2.5 2002/12/17 04:40:03 jon Exp $
  *
  * Copyright 2001 Jon Parise <jon@horde.org>
  *
@@ -17,6 +17,33 @@ function groupIsEditable($group)
     return PrefsUI::groupIsEditable($group);
 }
 
+function handle_category_management($updated)
+{
+    $actionID = Horde::getFormData('managementActionID');
+    $categoryID = Horde::getFormData('category_id');
+    $categoryName = Horde::getFormData('category_name');
+
+    switch ($actionID) {
+        case NAG_DELETE_CATEGORY:
+            Nag::deleteCategory($categoryID);
+            break;
+
+        case NAG_RENAME_CATEGORY:
+            Nag::renameCategory($categoryID, $categoryName);
+            break;
+
+        case NAG_ADD_CATEGORY:
+            Nag::addCategory($categoryName);
+            break;
+
+        default:
+            /* must have been the Save button */
+            return true;
+    }
+
+    /* return false so it stays on this page */
+    return false;
+}
 
 define('NAG_BASE', dirname(__FILE__));
 require_once NAG_BASE . '/lib/base.php';
@@ -41,5 +68,3 @@ PrefsUI::generateUI($group);
 
 $registry->shutdown();
 require NAG_TEMPLATES . '/common-footer.inc';
-
-?>
